// NpMsgrConfig.cpp: implementation of the CNpMsgrConfig class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "NpMsgrConfig.h"

#include "GBStream.h"

#if defined(WIN32)
#include "shlobj.h"
#endif

#include "GTCommonAPI.h"
#include "ConvertUTF.h"


#if defined(WIN32)
#define STR_PATH_SEPERATOR	"\\"
#else
#define STR_PATH_SEPERATOR	"/"
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CNpMsgrConfig::CNpMsgrConfig() : m_pNpMsgrUser(NULL)
{

}

CNpMsgrConfig::~CNpMsgrConfig()
{

}

void CNpMsgrConfig::InitModule(INpMsgrUser * pNpMsgrUser) 
{ 
	m_pNpMsgrUser = pNpMsgrUser; 
	
	LoadServerConfigFromCacheFile();
}


bool CNpMsgrConfig::LoadServerConfigFromCacheFile()
{
	char szPath[MAX_PATH+1];
	
	char szClientConfig[8096];	//now, data size is 2654.. enough to save
	
	if(!GetFilePath("ServerConfig", szPath) )
		return false;

	int nReadSize = GTCommonAPI::ReadDataFile(szPath, szClientConfig, sizeof(szClientConfig));
	
	if(nReadSize < 0)
		return false;

	m_strServerConfigData.assign( szClientConfig, nReadSize );
	
	return true;
}


void CNpMsgrConfig::UpdateServerConfig(char * pServerConfigData, USHORT usServerConfigDataLength)
{
	char szPath[MAX_PATH+1];
	if( GetFilePath("ServerConfig", szPath) )
	{
#if defined(WIN32)
		CUtil::WriteFile(szPath, pServerConfigData, usServerConfigDataLength);
#else
		GTCommonAPI::WriteDataFile(szPath, pServerConfigData, usServerConfigDataLength);
#endif		
	}

	m_strServerConfigData.assign( pServerConfigData, usServerConfigDataLength );
}

#define IK_SKIP_WHITESPACE( p )	\
	while( *p == ' ' || *p == '\t' ) p++;


DWORD GetMemoryProfileString(LPCSTR lpAppName,
							 LPCSTR lpKeyName,
							 LPCSTR lpDefault,
							 LPSTR lpReturnedString,
							 DWORD nSize,
							 LPCSTR lpContents
							 )
{
	string strAppNameKey	= string("[") + lpAppName + "]";
	string strKeyName		= lpKeyName;

	strcpy( lpReturnedString, lpDefault );
	
	const char * pNext = lpContents;

	bool bFindSection = false;
	do
	{
		IK_SKIP_WHITESPACE( pNext );

		if( *pNext == '\0' )
			return -1;

		if( !bFindSection )
		{
			if( *pNext == '[' )
			{
				if( strncmp( pNext, strAppNameKey.c_str(), strAppNameKey.length() ) == 0 )
				{
					pNext += strAppNameKey.length();
					if( *pNext != ' ' && *pNext != '\t' && *pNext != '\r' && *pNext != '\n' )
					{
						// ini  
						return -1;
					}

					bFindSection = true;
				}
			}
		}
		else
		{
			if( strncmp( pNext, strKeyName.c_str(), strKeyName.length() ) == 0 )
			{
				pNext += strKeyName.length();

				if( *pNext == ' ' || *pNext == '\t' || *pNext == '=' )
				{
					// find!!

					IK_SKIP_WHITESPACE( pNext );

					if( *pNext != '=')
					{
						//  
						return -1;
					}

					pNext++;
					IK_SKIP_WHITESPACE( pNext );

					if( *pNext == '"' )
						pNext++;

					unsigned int i = 0;
					for( i = 0 ; i <= nSize ; i++ )
					{
						if( *pNext != '\0' && *pNext != '\r' && *pNext != '\n' && *pNext != '"' )
                            lpReturnedString[i] = *pNext;
						else
							break;

						pNext++;
					}
					lpReturnedString[i] = '\0';

					return i;
				}
			}
		}

		if( *pNext == '\0' )
			return -1;

		pNext = strchr(pNext, '\n');
		if( pNext == NULL )
			return -1;

		pNext++;
	}
	while( *pNext != '\0' );

	return -1;
}
	
// INpMsgrConfig
bool CNpMsgrConfig::GetClientUserConfig(LPCSTR szConfigName, char szValue[], int nMax)
{
	return false;
}

bool CNpMsgrConfig::GetServerUserConfig(LPCSTR szConfigName, char szValue[], int nMax)
{
	return false;
}

bool CNpMsgrConfig::GetClientConfig(LPCSTR szConfigName, char szValue[], int nMax)
{
	szValue[0] = 0;
	if( strcmp("default_group_name", szConfigName) == 0 )
#if defined(WIN32)	
		strcpy(szValue, "⺻׷");
#else
		strcpy(szValue, "Default Group");
#endif	

	return false;
}

ULONG CNpMsgrConfig::GetServerConfigCacheSerial()
{
	char szValue[64];
	if( GetServerConfig( "CACHE", "SERIAL", szValue, 64 ) )
	{
		int n = atoi( szValue );
		return n;
	}

	return 0;
}

bool CNpMsgrConfig::GetServerConfig(LPCSTR szSection, LPCSTR szConfigName, char szValue[], int nMax)
{
	szValue[0] = 0;

#if defined(WIN32)
	char szPath[MAX_PATH+1];
	if( GetFilePath("ServerConfig", szPath) )
#endif			
	{
	
		/*
		The return value is the number of characters copied to the buffer, 
		not including the terminating null character.

		If neither lpAppName nor lpKeyName is NULL and the supplied destination buffer is 
		  too small to hold the requested string, 
		  the string is truncated and followed by a null character, 
		  and the return value is equal to nSize minus one.

		If either lpAppName or lpKeyName is NULL and the supplied destination buffer is 
		  too small to hold all the strings, 
		  the last string is truncated and followed by two null characters. 
		  In this case, the return value is equal to nSize minus two.
		*/

//		int n = GetPrivateProfileString( szSection, szConfigName, "", szValue, nMax, szPath );
		int n = GetMemoryProfileString( szSection, szConfigName, "", szValue, nMax, m_strServerConfigData.c_str() );
		if( n >= 0 )
			return true;
	}

	return false;
}


bool CNpMsgrConfig::GetFilePath(LPCSTR szPathName, char szPath[], int nMax)
{
	szPath[0] = '\0';

	if( stricmp( szPathName, "BuddyListCache" ) == 0 )
	{
		GetDirectoryPath("user_data", szPath, nMax );

		strcat( szPath, STR_PATH_SEPERATOR);
		strcat( szPath, "buddylist.ca" );
		return true;
	}
	else if( stricmp( szPathName, "BuddyProfileCache" ) == 0 )
	{
		GetDirectoryPath("user_data", szPath, nMax );

		strcat( szPath, STR_PATH_SEPERATOR);
		strcat( szPath, "buddyprofile.ca" );
		return true;
	}
	else if( stricmp( szPathName, "GroupListCache" ) == 0 )
	{
		GetDirectoryPath("user_data", szPath, nMax );
		
		strcat( szPath, STR_PATH_SEPERATOR);
		strcat( szPath, "grouplist.ca" );
		return true;
	}
	else if( stricmp( szPathName, "UserConfig" ) == 0 )
	{
		GetDirectoryPath("user_config", szPath, nMax );

		strcat( szPath, "config.cfg" );
		return true;
	}
	else if( stricmp( szPathName, "ServerConfig" ) == 0 )
	{
		GetDirectoryPath("root", szPath, nMax );

		strcat( szPath, STR_PATH_SEPERATOR);
		strcat( szPath, "server_config.ini" );
		return true;
	}
	else if( stricmp( szPathName, "CommonConfig" ) == 0 )
	{
		GetDirectoryPath("root", szPath, nMax );

		strcat( szPath, STR_PATH_SEPERATOR);
		strcat( szPath, "common_config.cfg" );
		return true;
	}
	else if( stricmp( szPathName, "RelayServerList" ) == 0 )
	{
		GetDirectoryPath("root", szPath, nMax );

		strcat( szPath, STR_PATH_SEPERATOR);
		strcat( szPath, "relayserver.ca" );
		return true;
	}
	else if( stricmp( szPathName, "error_xml_log" ) == 0 )
	{
		GetDirectoryPath("log", szPath, nMax );
		
		strcat( szPath, STR_PATH_SEPERATOR);
		strcat( szPath, "xml_error_log.xml" );
		return true;
	}
		
	ASSERT( false );

	return false;
}


bool CNpMsgrConfig::GetDirectoryPath(LPCSTR szPathName, char szPath[], int nMax)
{
	szPath[0] = '\0';

/*	GetModuleFileName( g_hInst, szPath, nMax );
	char * p = strrchr( szPath, '\\' );
	*p = '\0';
*/
	USES_UTF8_CONVERSION;
#if defined(WIN32)
	TCHAR szAppPath[MAX_PATH * 2] = {0,};
#else
	char szAppPath[MAX_PATH * 2] = {0,};
#endif

#if defined(WIN32)		
	if(SUCCEEDED(CUtil::NP_SHGetFolderPath(NULL, CSIDL_LOCAL_APPDATA, NULL, 0, szAppPath))) 
#else
	GTCommonAPI::GetLocalAppPath(szAppPath, sizeof(szAppPath));
#endif		
	{
		strcat(szPath, szAppPath);
		strcat( szPath, STR_PATH_SEPERATOR);
		strcat(szPath, "Ntalkbiz");
	}
	
	if( stricmp( szPathName, "root" ) == 0 )
	{
		return true;
	}
	else if( stricmp( szPathName, "log" ) == 0 )
	{
		strcat( szPath, STR_PATH_SEPERATOR);
		strcat( szPath, "log" );
	}
	else if( stricmp( szPathName, "tmp" ) == 0 )
	{
		strcat( szPath, STR_PATH_SEPERATOR);
		strcat( szPath, "tmp" );
	}
	else if( stricmp( szPathName, "user_data" ) == 0 )
	{
		ASSERT( wcslen( m_pNpMsgrUser->Id() ) > 0 );

		strcat( szPath, STR_PATH_SEPERATOR);
		strcat( szPath, "Profile" ); // Modified by jskim because of Folder policy Integration
		strcat( szPath, STR_PATH_SEPERATOR);
		strcat( szPath, W2U8(m_pNpMsgrUser->Id()) );
	}
	else if( stricmp( szPathName, "user_config" ) == 0 )
	{
		ASSERT( strlen( W2U8(m_pNpMsgrUser->Id()) ) > 0 );

		strcat( szPath, STR_PATH_SEPERATOR);
		strcat( szPath, "Profile" );
		strcat( szPath, STR_PATH_SEPERATOR);
		strcat( szPath, W2U8(m_pNpMsgrUser->Id()) );
	}
	else
	{
		return false;
	}

#if defined(WIN32)
	CUtil::VerifyDirectory( szPath );
#endif	
	return true;
}